@@ -1,5 +1,6 @@
 #This file is part of Tryton.  The COPYRIGHT file at the top level of
 #this repository contains the full copyright notices and license terms.
+import copy
 import tryton.rpc as rpc
 from record import Record
 from field import Field, O2MField
@@ -10,13 +11,17 @@ import tryton.common as common
 class Group(SignalEvent, list):
 
     def __init__(self, model_name, fields, window, ids=None, parent=None,
-            parent_name='', context=None, readonly=False,
-            parent_datetime_field=None):
+            parent_name='', child_name='', context=None, domain=None,
+            readonly=False, parent_datetime_field=None):
         super(Group, self).__init__()
+        if domain is None:
+            domain = []
+        self.domain = domain
         self.lock_signal = False
         self.__window = window
         self.parent = parent
         self.parent_name = parent_name
+        self.child_name = child_name
         self.parent_datetime_field = parent_datetime_field
         self._context = context or {}
         self.model_name = model_name
@@ -24,13 +29,13 @@ class Group(SignalEvent, list):
         self.load_fields(fields)
         self.current_idx = None
         self.load(ids)
-        self.record_removed = []
-        self.record_deleted = []
+        self.record_deleted, self.record_removed = [], []
         self.on_write = set()
         self.readonly = readonly
         if self._context.get('_datetime'):
             self.readonly = True
         self.__id2record = {}
+        self.__field_childs = None
 
     def __get_window(self):
         return self.__window
@@ -43,6 +48,7 @@ class Group(SignalEvent, list):
     window = property(__get_window, __set_window)
 
     def insert(self, pos, record):
+        assert record.group is self
         if pos >= 1:
             self.__getitem__(pos - 1).next[id(self)] = record
         if pos < self.__len__():
@@ -52,16 +58,17 @@ class Group(SignalEvent, list):
         super(Group, self).insert(pos, record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', pos))
+            self.signal('group-list-changed', ('record-added', record))
 
     def append(self, record):
+        assert record.group is self
         if self.__len__() >= 1:
             self.__getitem__(self.__len__() - 1).next[id(self)] = record
         record.next[id(self)] = None
         super(Group, self).append(record)
         self.__id2record[record.id] = record
         if not self.lock_signal:
-            self.signal('group-list-changed', ('record-added', -1))
+            self.signal('group-list-changed', ('record-added', record))
 
     def _remove(self, record):
         idx = self.index(record)
@@ -71,21 +78,23 @@ class Group(SignalEvent, list):
                         self.__getitem__(idx + 1)
             else:
                 self.__getitem__(idx - 1).next[id(self)] = None
+        if not self.lock_signal:
+            self.signal('group-list-changed', ('record-removed', record))
         super(Group, self).remove(record)
         del self.__id2record[record.id]
-        if not self.lock_signal:
-            self.signal('group-list-changed', ('record-removed', idx))
 
     def clear(self):
-        del self[:]
+        if not self.lock_signal:
+            for record in self[:]:
+                self.signal('group-list-changed', ('record-removed', record))
+                self.pop(0)
         if not self.lock_signal:
             self.signal('group-list-changed', ('group-cleared',))
-        self.record_removed = []
-        self.record_deleted = []
+        self.__id2record = {}
+        self.record_removed, self.record_deleted = [], []
 
     def move(self, record, pos):
-        self.lock_signal = True
-        if self.__len__() > pos:
+        if self.__len__() > pos >= 0:
             idx = self.index(record)
             self._remove(record)
             if pos > idx:
@@ -94,7 +103,6 @@ class Group(SignalEvent, list):
         else:
             self._remove(record)
             self.append(record)
-        self.lock_signal = False
 
     def __setitem__(self, i, value):
         super(Group, self).__setitem__(i, value)
@@ -108,7 +116,7 @@ class Group(SignalEvent, list):
         for name, attr in fields.iteritems():
             field = Field(attr['type'])
             attr['name'] = name
-            self.fields[name] = field(self, attr)
+            self.fields[name] = field(attr)
             if isinstance(self.fields[name], O2MField) \
                     and '_datetime' in self._context:
                 self.fields[name].context.update({
@@ -116,17 +124,26 @@ class Group(SignalEvent, list):
                     })
 
     def save(self):
+        saved = []
         for record in self:
-            saved = record.save()
-            self.writen(saved)
+            saved.append(record.save(force_reload=False))
+        return saved
 
-    def writen(self, ids):
+    @property
+    def root_group(self):
+        root = self
+        parent = self.parent
+        while parent:
+            root = parent.group
+        return root
+
+    def written(self, ids):
         if isinstance(ids, (int, long)):
             ids = [ids]
         ids = [x for x in self.on_write_ids(ids) or [] if x not in ids]
         if not ids:
             return
-        self.reload(ids)
+        self.root_group.reload(ids)
         return ids
 
     def reload(self, ids):
@@ -149,31 +166,34 @@ class Group(SignalEvent, list):
                 res += res2
         return list({}.fromkeys(res))
 
-    def load(self, ids, display=True, modified=False):
-        if not ids:
-            return True
-
-        old_ids = [x.id for x in self]
-        ids = [x for x in ids if x not in old_ids]
+    def load(self, ids, display=True, modified=False, id2record=None):
         if not ids:
             return True
 
         if len(ids) > 1:
             self.lock_signal = True
+
         new_records = []
         for id in ids:
+            if self.get(id):
+                continue
             new_record = Record(self.model_name, id, self.window,
-                    parent=self.parent, parent_name=self.parent_name, group=self)
+                group=self)
             self.append(new_record)
             new_records.append(new_record)
-            new_record.signal_connect(self, 'record-changed', self._record_changed)
-            new_record.signal_connect(self, 'record-modified', self._record_modified)
+            new_record.signal_connect(self, 'record-changed',
+                self._record_changed)
+            new_record.signal_connect(self, 'record-modified',
+                self._record_modified)
+
+        # Remove previously removed or deleted records
         for record in self.record_removed[:]:
             if record.id in ids:
                 self.record_removed.remove(record)
         for record in self.record_deleted[:]:
             if record.id in ids:
                 self.record_deleted.remove(record)
+
         if self.lock_signal:
             self.lock_signal = False
             self.signal('group-cleared')
@@ -198,26 +218,21 @@ class Group(SignalEvent, list):
     context = property(_get_context)
 
     def add(self, record, position=-1, modified=True):
-        if not record.group is self:
-            fields = {}
-            for i in record.group.fields:
-                fields[record.group.fields[i].attrs['name']] = \
-                        record.group.fields[i].attrs
-            self.add_fields(fields)
+        if record.group is not self:
+            record.signal_unconnect(record.group)
             record.group = self
-
+            record.window = self.window
+            record.signal_connect(self, 'record-changed', self._record_changed)
+            record.signal_connect(self, 'record-modified', self._record_modified)
         if position == -1:
             self.append(record)
         else:
             self.insert(position, record)
         self.current_idx = position
-        record.parent = self.parent
-        record.parent_name = self.parent_name
-        record.window = self.window
         if modified:
-            record.modified = True
-        record.signal_connect(self, 'record-changed', self._record_changed)
-        record.signal_connect(self, 'record-modified', self._record_modified)
+            record.modified_fields.setdefault('id')
+            record.signal('record-modified')
+        self.signal('group-changed', record)
         return record
 
     def set_sequence(self, field='sequence'):
@@ -226,13 +241,13 @@ class Group(SignalEvent, list):
             if record[field]:
                 if index >= record[field].get(record):
                     index += 1
-                    record[field].set(record, index, modified=True)
+                    record[field].set_client(record, index)
                 else:
                     index = record[field].get(record)
 
-    def new(self, default=True, domain=None, context=None, signal=True):
-        record = Record(self.model_name, None, self.window, group=self,
-                parent=self.parent, parent_name=self.parent_name)
+    def new(self, default=True, domain=None, context=None, signal=True,
+            obj_id=None):
+        record = Record(self.model_name, obj_id, self.window, group=self)
         record.signal_connect(self, 'record-changed', self._record_changed)
         record.signal_connect(self, 'record-modified', self._record_modified)
         if default:
@@ -244,18 +259,34 @@ class Group(SignalEvent, list):
             self.signal('group-changed', record)
         return record
 
-    def remove(self, record, remove=False, modified=True, signal=True):
+    def unremove(self, record, signal=True):
+        if record in self.record_removed:
+            self.record_removed.remove(record)
+        if record in self.record_deleted:
+            self.record_deleted.remove(record)
+        if signal:
+            record.signal('record-changed', record.parent)
+
+    def remove(self, record, remove=False, modified=True, signal=True,
+            force_remove=False):
         idx = self.index(record)
         if self[idx].id > 0:
             if remove:
+                if self[idx] in self.record_deleted:
+                    self.record_deleted.remove(self[idx])
                 self.record_removed.append(self[idx])
             else:
+                if self[idx] in self.record_removed:
+                    self.record_removed.remove(self[idx])
                 self.record_deleted.append(self[idx])
         if record.parent:
-            record.parent.modified = True
+            record.parent.modified_fields.setdefault('id')
+            record.parent.signal('record-modified')
         if modified:
-            record.modified = True
-        self._remove(self[idx])
+            record.modified_fields.setdefault('id')
+            record.signal('record-modified')
+        if not record.parent or self[idx].id <= 0 or force_remove:
+            self._remove(self[idx])
 
         if len(self):
             self.current_idx = min(idx, len(self) - 1)
@@ -339,12 +370,25 @@ class Group(SignalEvent, list):
         super(Group, self).destroy()
         self.__window = None
         self.parent = None
-        for field in self.fields.itervalues():
-            field.destroy()
         self.fields = {}
-        self.record_removed = None
-        self.record_deleted = None
+        self.record_deleted, self.record_removed = [], []
         self.__id2record = None
         for record in self:
             record.destroy()
         self[:] = []
+
+    def get_by_path(self, path):
+        'return record by path'
+        group = self
+        record = None
+        for child_name, id_ in path:
+            record = group.get(id_)
+            if not record:
+                return None
+            if not child_name:
+                continue
+            record[child_name]
+            group = record.value.get(child_name)
+            if not isinstance(group, Group):
+                return None
+        return record
