@@ -4,6 +4,7 @@ import gtk
 import gettext
 from tryton.common import message, TRYTON_ICON
 import tryton.rpc as rpc
+import tryton.common as common
 from interface import ParserView
 from tryton.action import Action
 from tryton.config import CONFIG
@@ -15,9 +16,10 @@ _ = gettext.gettext
 class ViewForm(ParserView):
 
     def __init__(self, window, screen, widget, children=None,
-            buttons=None, toolbar=None, notebooks=None, cursor_widget=''):
+            buttons=None, toolbar=None, notebooks=None, cursor_widget='',
+            children_field=None):
         super(ViewForm, self).__init__(window, screen, widget, children,
-                buttons, toolbar, notebooks, cursor_widget)
+                buttons, toolbar, notebooks, cursor_widget, children_field)
         self.view_type = 'form'
 
         for button in self.buttons:
@@ -35,13 +37,16 @@ class ViewForm(ParserView):
         scroll = gtk.ScrolledWindow()
         scroll.add(vp)
         scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
-        self.viewport = gtk.Viewport()
-        self.viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
-        self.viewport.add(scroll)
-        width, height = self.widget.size_request()
+        scroll.set_placement(gtk.CORNER_TOP_LEFT)
+        viewport = gtk.Viewport()
+        viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
+        viewport.add(scroll)
+        if isinstance(self.screen.window, gtk.Dialog):
+            width, height = self.widget.size_request()
+            vbox.set_size_request(width or -1, height or -1)
+        vbox.pack_start(viewport, expand=True, fill=True)
+
         self.widget = vbox
-        self.widget.set_size_request(width, height)
-        self.widget.pack_start(self.viewport, expand=True, fill=True)
 
         if toolbar and not CONFIG['client.modepda']:
             hbox = gtk.HBox(homogeneous=False)
@@ -56,11 +61,15 @@ class ViewForm(ParserView):
                     continue
 
                 for tool in toolbar[icontype]:
-                    iconstock = {
-                        'print': 'tryton-print',
-                        'action': 'tryton-executable',
-                        'relate': 'tryton-go-jump',
-                    }.get(icontype)
+                    if not tool['icon.rec_name']:
+                        iconstock = {
+                            'print': 'tryton-print',
+                            'action': 'tryton-executable',
+                            'relate': 'tryton-go-jump',
+                        }.get(icontype)
+                    else:
+                        iconstock = tool['icon.rec_name']
+                    common.ICONFACTORY.register_icon(iconstock)
 
                     if hasattr(gtk, 'MenuToolButton') and icontype == 'print':
                         tbutton = gtk.MenuToolButton(iconstock)
@@ -113,8 +122,9 @@ class ViewForm(ParserView):
                         'to be able to use the action button!'), self.window)
                 return False
             email = {}
-            if 'email' in action:
-                email = self.screen.current_record.expr_eval(action['email'])
+            if 'pyson_email' in action:
+                email = self.screen.current_record.expr_eval(
+                    action['pyson_email'])
                 if not email:
                     email = {}
             email['subject'] = action['name'].replace('_', '')
@@ -127,6 +137,10 @@ class ViewForm(ParserView):
                 message(_('You must select a record ' \
                         'to be able to use the relate button !'), self.window)
                 return False
+            if obj_id < 0:
+                message(_('You must save this record '
+                    'to be able to use the relate button!'), self.window)
+                return False
             encoder = PYSONEncoder()
             if 'pyson_domain' in act:
                 act['pyson_domain'] = encoder.encode(
@@ -144,7 +158,7 @@ class ViewForm(ParserView):
         value = Action._exec_action(act, self.window, data, {})
         if atype in ('print', 'action'):
             if self.screen:
-                self.screen.reload(writen=True)
+                self.screen.reload(written=True)
         return value
 
     def __getitem__(self, name):
@@ -206,7 +220,11 @@ class ViewForm(ParserView):
         record = self.screen.current_record
         if record:
             # Force to set fields in record
-            for field in record.group.fields:
+            # Get first the lazy one to reduce number of requests
+            fields = [(name, field.attrs.get('loading', 'eager'))
+                    for name, field in record.group.fields.iteritems()]
+            fields.sort(lambda x, y: cmp(y[1], x[1]))
+            for field, _ in fields:
                 record[field].get(record, check_load=False)
         for name, widgets in self.widgets.iteritems():
             field = None
